@@ -6,7 +6,8 @@
  */
 
 /*
- *  Copyright (C) 1997, 1998 Olivetti & Oracle Research Laboratory
+ *  Copyright (C) 2002 RealVNC Ltd.
+ *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
  *
  *  This is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -60,20 +61,18 @@ rfbSendRectEncodingCoRRE(cl, x, y, w, h)
     int x, y, w, h;
 {
     if (h > cl->correMaxHeight) {
-	rfbSendRectEncodingCoRRE(cl, x, y, w, cl->correMaxHeight  );
-	rfbSendRectEncodingCoRRE(cl, x, y + cl->correMaxHeight, w,
-				 h - cl->correMaxHeight);
-	return;
+      return (rfbSendRectEncodingCoRRE(cl, x, y, w, cl->correMaxHeight) &&
+              rfbSendRectEncodingCoRRE(cl, x, y + cl->correMaxHeight, w,
+                                       h - cl->correMaxHeight));
     }
 
     if (w > cl->correMaxWidth) {
-	rfbSendRectEncodingCoRRE(cl, x, y, cl->correMaxWidth, h);
-	rfbSendRectEncodingCoRRE(cl, x + cl->correMaxWidth, y,
-				 w - cl->correMaxWidth, h);
-	return;
+      return (rfbSendRectEncodingCoRRE(cl, x, y, cl->correMaxWidth, h) &&
+              rfbSendRectEncodingCoRRE(cl, x + cl->correMaxWidth, y,
+                                       w - cl->correMaxWidth, h));
     }
 
-    rfbSendSmallRectEncodingCoRRE(cl, x, y, w, h);
+    return rfbSendSmallRectEncodingCoRRE(cl, x, y, w, h);
 }
 
 
