@@ -5,6 +5,7 @@
  */
 
 /*
+ *  Copyright (C) 2002 RealVNC Ltd.
  *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
  *
  *  This is free software; you can redistribute it and/or modify
@@ -106,7 +107,7 @@ sendHextiles##bpp(cl, rx, ry, rw, rh)					      \
     int x, y, w, h;							      \
     int startUblen;							      \
     char *fbptr;							      \
-    CARD##bpp bg, fg, newBg, newFg;					      \
+    CARD##bpp bg=0, fg=0, newBg, newFg;					      \
     Bool mono, solid;							      \
     Bool validBg = FALSE;						      \
     Bool validFg = FALSE;						      \
@@ -120,7 +121,7 @@ sendHextiles##bpp(cl, rx, ry, rw, rh)					      \
 	    if (ry+rh - y < 16)						      \
 		h = ry+rh - y;						      \
 									      \
-	    if ((ublen + 1 + 16*16*(bpp/8)) > UPDATE_BUF_SIZE) {	      \
+	    if ((ublen + 1 + (2 + 16 * 16) * (bpp/8)) > UPDATE_BUF_SIZE) {    \
 		if (!rfbSendUpdateBuf(cl))				      \
 		    return FALSE;					      \
 	    }								      \
@@ -301,7 +302,7 @@ testColours##bpp(data,size,mono,solid,bg,fg)				      \
     CARD##bpp *bg;							      \
     CARD##bpp *fg;							      \
 {									      \
-    CARD##bpp colour1, colour2;						      \
+    CARD##bpp colour1=0, colour2=0;					      \
     int n1 = 0, n2 = 0;							      \
     *mono = TRUE;							      \
     *solid = TRUE;							      \
