@@ -5,6 +5,7 @@
  */
 
 /*
+ *  Copyright (C) 2002-2003 RealVNC Ltd.
  *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
  *
  *  This is free software; you can redistribute it and/or modify
@@ -536,7 +537,7 @@ rfbSpriteStoreColors (pMap, ndef, pdef)
 	{
 	    /* Direct color - match on any of the subfields */
 
-#define MaskMatch(a,b,mask) ((a) & (pVisual->mask) == (b) & (pVisual->mask))
+#define MaskMatch(a,b,mask) (((a) & (pVisual->mask)) == ((b) & (pVisual->mask)))
 
 #define UpdateDAC(plane,dac,mask) {\
     if (MaskMatch (pPriv->colors[plane].pixel,pdef[i].pixel,mask)) {\
@@ -1608,7 +1609,7 @@ rfbSpriteText (pDraw, pGC, x, y, count, chars, fontEncoding, textType, cursorBox
     unsigned long i;
     unsigned int  n;
     int		  w;
-    void   	  (*drawFunc)();
+    void   	  (*drawFunc)() = 0;
 
     Bool imageblt;
 
